/*
 *
 * X DLX Simulator
 *
 * The DLX is a 32-bit RISC machine based on the MIPS 2000 architecture.
 * The architecture includes 32 32-bit general purpose registers and a 
 * program counter.
 *
 * Floating point registers and an interrupt register are not implemented.
 *
 * The address space is 4 Gbytes, big-endian byte-addressable using 32-bit
 * addresses. All instructions must be properly aligned.
 *
 *
 * Cameron Humphries
 * Department of Computer Science
 * University of Adelaide
 * 1992
 *
 */
 
#include <stdio.h>
#include <signal.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>

#include "sim.h"

static void stop_sim(),step_sim(),run_sim(),reset_sim();

extern int num_kb;
extern int sim_pid;
extern void Write_Comm_Channel_Str();
extern void Write_Status_Message();
extern char *dlx_filename;
extern void Reset_Output();

/*
 * Create menu button for popup execute menu.
 */
void Create_Execute_Menu(Widget form)
{
  Widget w,execute_menu_popup;
  static Arg arg_list[] = {
    {XtNlabel, (XtArgVal)"EXECUTE"},
    {XtNfromHoriz, (XtArgVal)NULL},
    {XtNmenuName, (XtArgVal)"Execute_Menu_Popup"},
  };
  static Arg menu_entry_args[] = {
    {XtNlabel, (XtArgVal)NULL},
    {XtNcallback, (XtArgVal)NULL},
  };
  static XtCallbackRec stop_cbks[] = {
    {stop_sim, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec step_cbks[] = {
    {step_sim, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec run_cbks[] = {
    {run_sim, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec reset_cbks[] = {
    {reset_sim, NULL},
    {NULL,NULL}
  };
  
  /* Create Popup widget */
  execute_menu_popup = XtCreatePopupShell("Execute_Menu_Popup",simpleMenuWidgetClass,
					  form,NULL,0);
  
  /* Create Menu entries */
  menu_entry_args[0].value = (XtArgVal)"Step";
  menu_entry_args[1].value = (XtArgVal)step_cbks;
  w = XtCreateManagedWidget("EM_Run",smeBSBObjectClass,execute_menu_popup,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  menu_entry_args[0].value = (XtArgVal)"Run";
  menu_entry_args[1].value = (XtArgVal)run_cbks;
  w = XtCreateManagedWidget("EM_Run",smeBSBObjectClass,execute_menu_popup,
			    menu_entry_args,XtNumber(menu_entry_args));

  menu_entry_args[0].value = (XtArgVal)"Stop";
  menu_entry_args[1].value = (XtArgVal)stop_cbks;
  w = XtCreateManagedWidget("EM_Run",smeBSBObjectClass,execute_menu_popup,
			    menu_entry_args,XtNumber(menu_entry_args));

  menu_entry_args[0].value = (XtArgVal)"Reset";
  menu_entry_args[1].value = (XtArgVal)reset_cbks;
  w = XtCreateManagedWidget("EM_Run",smeBSBObjectClass,execute_menu_popup,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  /* Create Menu button */
  arg_list[1].value = (XtArgVal)XtNameToWidget(form,"File_Menu");
  w = XtCreateManagedWidget("Execute_Menu",menuButtonWidgetClass,form,
			    arg_list,XtNumber(arg_list));
  
  return;
}

static void stop_sim(Widget w,XtPointer client_data, XtPointer call_data)
{
  (void)Write_Status_Message("Stop simulator"); 

  kill(sim_pid,SIGINT);

  return;
}

static void step_sim(Widget w,XtPointer client_data, XtPointer call_data)
{
  (void)Write_Status_Message("Step simulator"); 
  /*
   * Write down pipe "step"
   */
  Write_Comm_Channel_Str("s");

  return;
}

static void run_sim(Widget w,XtPointer client_data, XtPointer call_data)
{
  (void)Write_Status_Message("Run Simulator"); 
  /*
   * Write down pipe "run"
   */
  Write_Comm_Channel_Str("r");

  return;
}

static void reset_sim(Widget w,XtPointer client_data, XtPointer call_data)
{
  (void)Write_Status_Message("Reset Simulator"); 

  kill(sim_pid,SIGINT);

  bzero(mem,num_kb*1024+34*sizeof(int));
  if (dlx_filename)
    load(dlx_filename);
  Update_all();
  Reset_Output();
  Write_Status_Message("READY");
  
  return;
}

